//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//
// Class          : simulation.MoveableSimulton
//
/// Author         : Richard E. Pattis
//                  Computer Science Department
//                  School of Information and Computer Science
//                  University of California at Irvine
//                  Irvine, CA 92617-3435
//                  e-mail: pattis@ics.uci.edu
//
// Maintainer     : Author
//
//
// Description:
//
//   Moveable is another important abstract class in the simulation
// package; it extends the Simulton class with methods/fields for moving
// (representing its velocity by a speed and an angle -both doubles). It
// define mutators for changing the speed/angle and accessors that return
// the current speed/angle. It defines a very simple update method, which
// moves the object (according to its current speed/angle) and
// automatically bounces object off the left, right, top, or bottom wall.
// The update methods in subclasses that require this behavior, and more,
// can use super to call this method (before/after doing other things).
//
//   The speed unit is pixels/update (so small numbers, e.g., < 10 work
// fine). The angle unit is radians. If you are unfamiliar with this
// unit, examine the diagram below in which all units are in radians. 
//
//                   pi/2
//                     |
//                     |
//                     | 
//                     |
//                     |
// pi (or -pi) --------+-------- 0 (or 2pi)
//                     |
//                     |
//                     |
//                     |
//                     |
//                  3pi/2 (or -pi/2)
//
//   Note that "no access modifier" for a member means that the member is
// package-friendly: this means that the member is public to all other
// classes in the simulation package, but private elsewhere.
//
// Future Plans   : Javadoc Comments
//
// Program History:
//  10/01/04: R. Pattis - Operational for 15-200
//
//
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////


package simulation;


import java.awt.Dimension;
import java.awt.Point;
import java.awt.Rectangle;



public abstract class MoveableSimulton extends Simulton {

  MoveableSimulton (int x, int y)
  {super(x,y);}



  //Mutators
  
  void setVelocity (double speed, double angle)
  {
    this.speed = speed;
    this.angle = angle;
  }
  
  
  void setSpeed (double speed)
  {this.speed = speed;}
  
  
  void setAngle (double angle)
  {this.angle = angle;}
  
  
  void randomizeAngle ()
  {this.angle = 2*Math.PI*Math.random();}
  
  
  void bounce(double barrierAngle)
  {angle = 2*barrierAngle - angle;}
  
  
  //Allthough move and wallBounce are separate methods,
  //wallBounce is typically called immediately following move.
  void move ()
  {
    //Update x,y coordinate by translation.
    //Note: - for y, because of coordinate system.
    getLocation().translate( (int)(speed*Math.cos(angle)) ,
                             (int)(-speed*Math.sin(angle))  );
  }
  
  
  //Abstract (in Simulton) now concrete
  void wallBounce (Model m)
  {
    Point p = getLocation();
    
    //Check for bounce
		Dimension enclosingBox = m.getEnclosingBox();
    Rectangle objectBox    = getOuterBox();
		int       leftmostX    = objectBox.x;
		int       rightmostX   = objectBox.x + objectBox.width;
		int       topmostY     = objectBox.y;
		int       bottommostY  = objectBox.y + objectBox.height;

		if (leftmostX < 0) {
		   bounce(Math.PI/2.);
		   p.translate(-2*leftmostX,0);
		}else if (rightmostX > enclosingBox.width) {
		   bounce(Math.PI/2.);
		   p.translate(2*(enclosingBox.width-rightmostX),0);
		}

		if (topmostY< 0) {
		   bounce(0.);
		   p.translate(0,-2*topmostY);
		}else if (bottommostY > enclosingBox.height) {
		   bounce(0.);
		   p.translate(0,2*(enclosingBox.height-bottommostY));
		}
  } 
  
  
  
  //Accessors
  
  double getSpeed ()
  {return speed;}
  
  
  double getAngle ()
  {return angle;}

  
  
  //Instance variables
    
  private double speed, angle;
}
