//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//
// Class          : simulation.Simulton
//
// Author         : Richard E. Pattis
//                  Computer Science Department
//                  School of Information and Computer Science
//                  University of California at Irvine
//                  Irvine, CA 92617-3435
//                  e-mail: pattis@ics.uci.edu
//
// Maintainer     : Author
//
//
// Description:
//
//   Simulton is the main abstract class in the simulation package. It
// represents a superclass for objects that are simulatable: each of
// these objects must have a location in space and each should be able
// to update/display itself, and answer questions related to Bounding
// Boxes that can be used to help determine collisions among simulatable
// objects. The two abstract methods specify its behavior (update) and
// retrieve its image (getImage); 
//
//   Note that "no access modifier" for a member means that the member is
// package-friendly: this means that the member is public to all other
// classes in the simulation package, but private elsewhere.
//
// Future Plans   : Javadoc
//
// Program History:
//   10/010/04: R. Pattis - Operational for 15-200
//
//
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////


package simulation;


import java.awt.Graphics;
import java.awt.Point;
import java.awt.Rectangle;



public abstract class Simulton {

  Simulton (int x, int y)
  {centerLocation = new Point(x,y);}
  
  
  
  //Abstract Methods
  
  abstract void        update  (Model m);
  abstract Displayable getImage();



  void display (Graphics g)
  {getImage().displayAt(centerLocation,g);}
  
  
  //These are written concretely based on the
  //simple methods a Displayable object defines.
  //Override these methods in subclass if more
  //detailed/accurate information is available
  //to a Simulton subclass
  
  boolean encloses (Point p)
  {return getImage().enclosesAt(centerLocation,p);}
  
  Rectangle getOuterBox ()
  {return getImage().getOuterBoxAt(getLocation());}
  
  Rectangle getInnerBox ()
  {return getImage().getInnerBoxAt(getLocation());}
  
  
  //Override this method of the Simulton subclass
  //  wants to bounce (otherwise do not override it
  //  and this one will be executed with is no effect)
  void bounce (double angle)
  {}
   
   
   
  //Accessors
  
  Point getLocation()
  {return centerLocation;}


  void setLocation(Point p)
  {centerLocation.setLocation(p);}


  boolean cannotOverlap(Simulton o)
  {return !getOuterBox().intersects(o.getOuterBox());}


  boolean mustOverlap(Simulton o)
  {return getInnerBox().intersects(o.getInnerBox());}


  boolean mayOverlap(Simulton o)
  {return !cannotOverlap(o);}



  //Instance variable
  //Although centerLocation is final, you can call mutators defined in
  //  the Point class to change the state of the object that it refers
  //  to. To get this object from outside the class, call the
  //  getLocation method.
  
  private final Point centerLocation;
}



