//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//
// Class          : simulation.View
//
// Author         : Richard E. Pattis
//                  Computer Science Department
//                  School of Information and Computer Science
//                  University of California at Irvine
//                  Irvine, CA 92617-3435
//                  e-mail: pattis@ics.uci.edu
//
// Maintainer     : Author
//
//
// Description:
//
//   This View for the Pinball package comprises two JPanels. The North
// one contains the controls (and a JText field); the Center one is for
// drawing; its paint method calls the model to "draw" all the pinball
// objects in their positions. By the Controller repeatedly "updating"
// the objects and calling repaint, the moveable objects are animated.
//
//   Note that "no access modifier" means that the method is package
// friendly: this means the member is public to all other classes in
// the Pinball package, but private elsewhere.
//
// Future Plans   : More Comments
//
// Program History:
//  10/20/01: R. Pattis - Operational for 15-100
//
//
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////



package simulation;


import javax.swing.BoxLayout;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;

import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;



public class View extends JFrame
{

  //Trivial constructor (could be automatically supplied by Java)
	public View()
	{}
	
  
  //Refer to the model (used in build to initialize it) 
	public void addModel(Model m)
	{model = m;}

	
  //Refer to the Controller (used in build to start animation)
	public void addController(Controller c)
	{controller = c;}

	
  //build does the heavy lifting; it builds the view, populating it
  //  with the appropriate JPanel
  //GUI applications have lots of little details to specify to make
  //  them look nice, and this method is in charge of them all
  public void build()
  {
	  //When a window close icon is pressed, exit the entire program
		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e)
			{
				System.exit(0);
			}
		});

		//Create a font for all the buttons (save it in a field, so it can be
		//  referred to in methods called subsequently: buttonSetup)
		buttonFont = new Font("Monospaced", Font.BOLD, 11);

    //Title the JFrame and scale/set an icon for all the balls
    setTitle("Simulation");
    
    //Setup the JPanel for the controls: seven JButtons and one JLabel
    JPanel controls = new JPanel();
    controls.setLayout(new BoxLayout(controls,BoxLayout.X_AXIS));
    buttonSetup(controls,controller.getResetButton()           , "Reset");
    buttonSetup(controls,controller.getObjectButton("ball")    , "Ball");
    buttonSetup(controls,controller.getObjectButton("floater") , "Floater");
    buttonSetup(controls,controller.getObjectButton("hole")    , "Hole");
    buttonSetup(controls,controller.getObjectButton("pulsator"), "Pulsator");
    buttonSetup(controls,controller.getObjectButton("hunter")  , "Hunter");
    buttonSetup(controls,controller.getObjectButton("special") , "Special");
    buttonSetup(controls,controller.getObjectButton("remove")  , "Remove");
    points = new JLabel("      updates/Simultons =   0");
    points.setFont(buttonFont);
    controls.add(points);
    
    animationPanel = new AnimationPanel(model);
    
    //Controller supplies Listener for mouse clicks on the panel
    animationPanel.addMouseListener(controller.getBallPanelMouseController());
    
    //Add JPanels for the controls (to the North) and the BallPanel (to
    //  the Center) to the content pane of the JFrame
    Container contentPane = getContentPane();
    contentPane.add(controls, "North");
    contentPane.add(animationPanel,"Center");
    
    //Controller supplies Listener for key presses in this JFrame (not in
    //  JPanel!)
    addKeyListener(controller.getBallPanelKeyController());

    //Figure out the sizes for everything, initialize the model, and
    //  start animating it
    pack();
    controller.animate();
  }



  //Called only above, to set the attributes of the buttons (their label,
  //  font) and add them to the GUI Panel
  private void buttonSetup(JPanel  panelForButtons,
                           JButton b,
                           String  bLabel)
  {
		b.setText(bLabel);
		b.setFont(buttonFont);
		panelForButtons.add(b);
  }
  
  
  String padTo(String s, int size)
  {return s.length() >= size ? s : padTo(" "+s,size);}
  
  //First set the Cycles, then paint all the JPanels in this JFrame
  public void paint(Graphics g)
  {
    points.setText(padTo(""+model.getUpdateCount(),5) + " updates/Simultons = " + model.getSimultonCount());
    super.paint(g);
  }
  
 
  
  //The model needs this information to determine bouncing behavior
  //It changes if the user changes the frame size.
  Dimension getBox()
  {return animationPanel.getSize();}



  //Instance variables
  
  private Model      model;
  private Controller controller;
  
	private Font            buttonFont; 
  private AnimationPanel  animationPanel;
  private JLabel          points;
 }
